@@ -14,3 +14,12 @@ class Payment(APIView):
             'msg': 'OK',
             "data": payment
         })
+
+    def refund(self):
+        params = parse_data(self)
+        payment = PaymentService.refund(params)
+        return JsonResponse({
+            'err': 0,
+            'msg': 'OK',
+            "data": payment
+        })
