@@ -10,7 +10,6 @@ class PaymentWechat(APIView):
     def get_user_info(self):
         code = self.GET.get('code', 0)
         wxpay_params = PaymentWechatService.get_user_info(code)
-
         return JsonResponse({
             'err': 0,
             'msg': 'OK',
@@ -36,11 +35,21 @@ class PaymentWechat(APIView):
             'msg': 'OK',
             'data': wxpay_params
         })
+        # 扫码支付
+
+    def payment_red_envelopes(self):
+        params = self.POST
+        wxpay_params = PaymentWechatService.payment_red_envelopes(params)
+        return JsonResponse({
+            'err': 0,
+            'msg': 'OK',
+            'data': wxpay_params
+        })
+        # 回调接口
 
-    # 回调接口
     def callback(self):
-        _xml = self.body
-        wxpay_params = PaymentWechatService.callback(_xml)
+        body = self.body
+        wxpay_params = PaymentWechatService.callback_old(body)
         return HttpResponse(wxpay_params)
         # return JsonResponse(wxpay_params)
         # return JsonResponse({
@@ -48,3 +57,14 @@ class PaymentWechat(APIView):
         #     'msg': 'OK',
         #     'data': wxpay_params
         # })
+
+    # 回调接口
+    def callback_v3(self):
+        body = self.body
+        headers = {}
+        headers.update({'Wechatpay-Signature': self.META.get('HTTP_WECHATPAY_SIGNATURE')})
+        headers.update({'Wechatpay-Timestamp': self.META.get('HTTP_WECHATPAY_TIMESTAMP')})
+        headers.update({'Wechatpay-Nonce': self.META.get('HTTP_WECHATPAY_NONCE')})
+        headers.update({'Wechatpay-Serial': self.META.get('HTTP_WECHATPAY_SERIAL')})
+        wxpay_params = PaymentWechatService.callback(headers, body)
+        return JsonResponse(wxpay_params)
