@@ -1,12 +1,13 @@
 import random
 
 from django.http import HttpResponse
+from django.utils import timezone
 
 from config.config import Config
 # from ..services.payment_service import PaymentService
-from ..utils.common import get_domain
+from xj_enroll.service.enroll_services import EnrollServices
 from ..utils.alipay_utils import *
-from django.utils import timezone
+from ..utils.common import get_domain
 
 gatway = Config.getIns().get("xj_payment", "GATEWAY", "https://openapi.alipaydev.com/gateway.do")
 
@@ -92,7 +93,9 @@ class PaymentAlipayService:
         print("异步通知验证状态: ", success)
         if success:
             # 此处写支付验签成功修改订单状态相关业务逻辑
-            PaymentService.create(callback_data)
+            # PaymentService.create(callback_data)
+            data, err = EnrollServices.pay_call_back(callback_data["order_no"])
+            print("pay_call_back:", data, err)
             return HttpResponse('success')  # 返回success给支付宝服务器, 若支付宝收不到success字符会重复发送通知
         return HttpResponse('fail')
 
