@@ -69,6 +69,9 @@ class PaymentService:
 
             payment = PaymentWechatService.payment_applets_pay(data)
 
+        elif params['payment_method'] == "appletsv3":  # 微信小程序支付v3
+            payment = PaymentWechatService.payment_applets_pay_v3(data)
+
         elif params['payment_method'] == "balance":  # 余额支付
             payment = PaymentWechatService.payment_balance_pay(data)
 
@@ -78,9 +81,12 @@ class PaymentService:
     @staticmethod
     def refund(params):
         data = params
-        data['total_fee'] = float(params['total_amount']) * 100  # 元转分
+        data['transaction_id'] = params['transaction_id']  # 支付单号
         data['refund_fee'] = float(params['refund_amount']) * 100  # 元转分
         data['out_trade_no'] = timezone.now().strftime('%Y%m%d%H%M%S') + ''.join(
             map(str, random.sample(range(0, 9), 4)))
-        payment = PaymentWechatService.payment_refund(data)
+        # 支付方式检查
+        if params['payment_method'] == "WECHAT":  # 微信退款
+            payment = PaymentWechatService.payment_refund(data)
+
         return payment
